#include <err.h>
#include <pcap.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h>
#include <time.h>
#include <unistd.h>

#define	PCAP_TIMEOUT	1000
#define	PCAP_SNAPLEN	1500

char	*device = NULL;
int	promisc = 1;

pcap_t	*pd = NULL;
int	done = 0;

void shutdown (int sig) {
	done = 1;
	pcap_close(pd);
	pd = NULL;
}

void hexdump (void *src, int len) {
	int i;
	char hex[80],ascii[40];
	unsigned char *cp = (char *)src;

	for (i = 0; i < len; i++) {
		if (!(i % 16)) {
			if (i) { printf("%-54s %s\n",hex,ascii); }
			sprintf(hex,"%04x: ",i);
			strcpy(ascii,"................");
		}
		sprintf((hex + 6) + (3 * (i % 16)),"%02x ",cp[i]);
		if (cp[i] > 32 && cp[i] < 127) { ascii[i % 16] = cp[i]; }
	}
	printf("%-54s %s\n",hex,ascii);
}

int main(int argc, char *argv[]) {
	char			errbuf[PCAP_ERRBUF_SIZE];
	const u_char		*packet;
	struct pcap_pkthdr	hdr;
	int			i, count = 0;
        char			time_buf[128];
	struct tm		*loctime;

	/* Process command line options */
	while ((i = getopt(argc,argv,"pi:")) != -1) {
		switch(i) {
		case 'p':
			promisc = 0;
			break;
		case 'i':
			device = strdup(optarg);
			break;
		default:
			errx(EX_USAGE,"Unrecognised command line option");
		}
	}

	/* If a device wasn't specified let pcap suggest one */
	if (device == NULL) {
		if ((device = pcap_lookupdev(errbuf)) == NULL)
			errx(EX_UNAVAILABLE,"%s",errbuf);
	}

	/* stdout line buffered please... */
	if (setvbuf(stdout,NULL, _IOLBF,0) == EOF)
		warnx("Unable to make stdout linebuffered!");

	/* Catch SIGINT so we can shutdown cleanly... */
	if (signal(SIGINT,shutdown) == SIG_ERR)
		err(EX_OSERR,"signal handler");

	/* Startup pcap... */
	if ((pd = pcap_open_live(device,PCAP_SNAPLEN,promisc,PCAP_TIMEOUT,errbuf)) == NULL)
		errx(EX_IOERR,"%s",errbuf);

	while(!done) {
		/* Snarf a packet.  If pcap times out, try again... */
		if ((packet = pcap_next(pd,&hdr)) == NULL)
			continue;

		/* Display packet... */
		loctime = localtime(&hdr.ts.tv_sec);		
		strftime(time_buf,sizeof(time_buf)-1,"%k:%M:%S",loctime); 
		printf("Packet %04d [%s.%03ld] (%d/0x%x)\n",
			++count,time_buf,hdr.ts.tv_usec / 1000,hdr.len,hdr.len);
		hexdump((void *)packet,hdr.caplen);
	}

	/* Shutdown pcap if our signal handler hasn't done it already... */
	if (pd != NULL)
		pcap_close(pd);

	/* All done.  Bye, bye... */
	return(EX_OK);
}
